@@ -63,7 +63,10 @@ static struct {
 		{ NULL, NULL, NULL } },
 	{ "timeout_enable", NULL, NULL,
 		CONFIG_ROOT "/timeout_enable",
-		{ "timeout_slide", "timeout_spin", "timeout_label" } }
+		{ "timeout_slide", "timeout_spin", "timeout_label" } },
+	{ "feature_state_change_beep", NULL, NULL,
+		CONFIG_ROOT "/feature_state_change_beep",
+		{ NULL, NULL, NULL } }
 };
 
 static struct {
@@ -77,14 +80,14 @@ static struct {
 } const ranges [] = {
 	{ "bouncekeys_delay_slide",	"bouncekeys_delay_spin",     300,  10,  900,   10,
 	  CONFIG_ROOT "/bouncekeys_delay" },
-	{ "slowkeys_delay_slide",	"slowkeys_delay_spin",	     300,  10,  900,   10,
+	{ "slowkeys_delay_slide",	"slowkeys_delay_spin",	     300,  10,  500,   10,
 	  CONFIG_ROOT "/slowkeys_delay" },
 	  /* WARNING anything larger than approx 512 seems to loose all keyboard input */
-	{ "mousekeys_max_speed_slide",	"mousekeys_max_speed_spin",  300,  10, 500,   20,
+	{ "mousekeys_max_speed_slide",	"mousekeys_max_speed_spin",  300,  10, 500,   10,
 	  CONFIG_ROOT "/mousekeys_max_speed" },
-	{ "mousekeys_accel_time_slide",	"mousekeys_accel_time_spin", 300,  10, 3000,  100,
+	{ "mousekeys_accel_time_slide",	"mousekeys_accel_time_spin", 300,  10, 3000,  10,
 	  CONFIG_ROOT "/mousekeys_accel_time" },
-	{ "mousekeys_init_delay_slide",	"mousekeys_init_delay_spin", 300,  10, 5000,  100,
+	{ "mousekeys_init_delay_slide",	"mousekeys_init_delay_spin", 300,  10, 5000,  10,
 	  CONFIG_ROOT "/mousekeys_init_delay" },
 	{ "timeout_slide",	"timeout_spin",			     200,  10,  500,   10,
 	  CONFIG_ROOT "/timeout" },
@@ -106,7 +109,9 @@ set_sensitive (GladeXML *dialog, char const *name, gboolean state)
 static void
 cb_feature_toggled (GtkToggleButton *btn, gpointer feature_index)
 {
-	gboolean const state = gtk_toggle_button_get_active (btn);
+	gboolean const state =
+		(GTK_WIDGET_STATE (btn) != GTK_STATE_INSENSITIVE) &&
+		gtk_toggle_button_get_active (btn);
 	GladeXML *dialog = g_object_get_data (G_OBJECT (btn), "dialog");
 	int feature, i;
 
@@ -148,7 +153,6 @@ setup_simple_toggles (GladeXML *dialog, GConfChangeSet *changeset)
 		char const *gconf_key;
 		char const *checkbox;
 	} const simple_toggles [] = {
-		{ CONFIG_ROOT "/feature_state_change_beep","feature_state_change_beep" },
 		{ CONFIG_ROOT "/bouncekeys_beep_reject",  "bouncekeys_beep_reject" },
 
 		{ CONFIG_ROOT "/slowkeys_beep_press",      "slowkeys_beep_press" },
@@ -231,67 +235,74 @@ cb_master_enable_toggle (GtkToggleButton *btn, GladeXML *dialog)
 {
 	int i = G_N_ELEMENTS (features);
 	gboolean flag = gtk_toggle_button_get_active (btn);
+	GtkWidget *w;
 
-	while (i-- > 0)
-		gtk_widget_set_sensitive (WID (features [i].checkbox), flag);
+	while (i-- > 0) {
+		w = WID (features [i].checkbox);
+		gtk_widget_set_sensitive (w, flag);
+		cb_feature_toggled (GTK_TOGGLE_BUTTON (w), GINT_TO_POINTER (i));
+	}
 }
 
 static void
 setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 {
 	GtkWidget *master_enable = WID ("master_enable");
+	setup_images (dialog);
+	setup_ranges (dialog, changeset);
+	setup_toggles (dialog, changeset);
+	setup_simple_toggles (dialog, changeset);
+
 	g_signal_connect (master_enable,
 		"toggled",
 		G_CALLBACK (cb_master_enable_toggle), dialog);
 	gconf_peditor_new_boolean (changeset,
 		CONFIG_ROOT "/enable",
 		GTK_WIDGET (master_enable), NULL);
-
-	setup_images (dialog);
-	setup_ranges (dialog, changeset);
-	setup_toggles (dialog, changeset);
-	setup_simple_toggles (dialog, changeset);
 }
 
 /*******************************************************************************/
 
-static void
+static gboolean
 xrm_get_bool (GConfClient *client, XrmDatabase *db, char const *gconf_key,
 	      char const *res_str, char const *class_str)
 {
 	XrmValue  resourceValue;
 	char	 *res;
 
-	if (XrmGetResource (*db, res_str, class_str, &res, &resourceValue))
+	if (!XrmGetResource (*db, res_str, class_str, &res, &resourceValue))
+		return FALSE;
 		gconf_client_set_bool (client, gconf_key,
 			!g_ascii_strcasecmp (resourceValue.addr, "True"), NULL);
+
+	return TRUE;
 }
 
-static void
+static gboolean
 xrm_get_int (GConfClient *client, XrmDatabase *db, char const *gconf_key,
-	     char const *res_str, char const *class_str)
+	     char const *res_str, char const *class_str, float scale)
 {
 	XrmValue	resourceValue;
 	char		*res;
-	int		value;
+	int		value, log_scale;
 	char		resource [256];
 
 	snprintf (resource, sizeof (resource), "%s.value", res_str);
 	if (!XrmGetResource (*db, resource, class_str, &res, &resourceValue))
-		return;
+		return FALSE;
 	value = atoi (resourceValue.addr);
 
-#if 0
-	{
-	int decimal;
 	snprintf (resource, sizeof (resource), "%s.decimalPoints", res_str);
 	if (!XrmGetResource (*db, resource, class_str, &res, &resourceValue))
-		return;
-	decimal = atoi (resourceValue.addr);
-	}
-#endif
+		return FALSE;
+	log_scale = atoi (resourceValue.addr);
+
+	while (log_scale-- > 0)
+		scale /= 10.;
 
 	gconf_client_set_int (client, gconf_key, value, NULL);
+
+	return TRUE;
 }
 
 /* This loads the current users XKB settings from their file */
@@ -301,6 +312,7 @@ load_CDE_file (GtkFileSelection *fsel)
 	char const *file = gtk_file_selection_get_filename (fsel);
 	GConfClient *client;
 	XrmDatabase  db;
+	gboolean found = FALSE;
 
 	if (!(db = XrmGetFileDatabase (file))) {
 		GtkWidget *warn = gtk_message_dialog_new (
@@ -316,43 +328,62 @@ load_CDE_file (GtkFileSelection *fsel)
 	}
 
 	client = gconf_client_get_default ();
-	xrm_get_bool (client, &db, CONFIG_ROOT "/enable",
-		"*EnableAccessXToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/feature_state_change_beep",
-		"*SoundOnOffToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/timeout_enable",
-		"*TimeOutToggle.set",		"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/stickykeys_enable",
-		"*StickyKeysToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/mousekeys_enable",
-		"*MouseKeysToggle.set",		"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/togglekeys_enable",
-		"*ToggleKeysToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/slowkeys_enable",
-		"*SlowKeysToggle.set",		"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/bouncekeys_enable",
-		"*BounceKeysToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/stickykeys_modifier_beep",
-		"*StickyModSoundToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/stickykeys_two_key_off",
-		"*StickyTwoKeysToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/slowkeys_beep_press",
-		"*SlowKeysOnPressToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_bool (client, &db, CONFIG_ROOT "/slowkeys_beep_accept",
-		"*SlowKeysOnAcceptToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
-	xrm_get_int  (client, &db, CONFIG_ROOT "/timeout",
-		"*TimeOutScale",		"AccessX*XmScale");
-	xrm_get_int  (client, &db, CONFIG_ROOT "/mousekeys_max_speed",
-		"*MouseMaxSpeedScale",		"AccessX*XmScale");
-	xrm_get_int  (client, &db, CONFIG_ROOT "/mousekeys_accel_time",
-		"*MouseAccelScale",		"AccessX*XmScale");
-	xrm_get_int  (client, &db, CONFIG_ROOT "/mousekeys_init_delay",
-		"*MouseDelayScale",		"AccessX*XmScale");
-	xrm_get_int  (client, &db, CONFIG_ROOT "/slowkeys_delay",
-		"*KRGSlowKeysDelayScale",	"AccessX*XmScale");
-	xrm_get_int  (client, &db, CONFIG_ROOT "/bouncekeys_delay",
-		"*KRGDebounceScale",		"AccessX*XmScale");
+	gconf_client_set_bool (client, CONFIG_ROOT "/enable", TRUE, NULL);
 
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/feature_state_change_beep",
+		"*SoundOnOffToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/timeout_enable",
+		"*TimeOutToggle.set",		"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/stickykeys_enable",
+		"*StickyKeysToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/mousekeys_enable",
+		"*MouseKeysToggle.set",		"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/togglekeys_enable",
+		"*ToggleKeysToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/slowkeys_enable",
+		"*SlowKeysToggle.set",		"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/bouncekeys_enable",
+		"*BounceKeysToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/stickykeys_modifier_beep",
+		"*StickyModSoundToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/stickykeys_two_key_off",
+		"*StickyTwoKeysToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/slowkeys_beep_press",
+		"*SlowKeysOnPressToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/slowkeys_beep_accept",
+		"*SlowKeysOnAcceptToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
+	found |= xrm_get_int  (client, &db, CONFIG_ROOT "/timeout",
+		"*TimeOutScale",		"AccessX*XmScale", 60);
+	found |= xrm_get_int  (client, &db, CONFIG_ROOT "/mousekeys_max_speed",
+		"*MouseMaxSpeedScale",		"AccessX*XmScale", 1);
+	found |= xrm_get_int  (client, &db, CONFIG_ROOT "/mousekeys_accel_time",
+		"*MouseAccelScale",		"AccessX*XmScale", 1);
+	found |= xrm_get_int  (client, &db, CONFIG_ROOT "/mousekeys_init_delay",
+		"*MouseDelayScale",		"AccessX*XmScale", 1);
+	found |= xrm_get_int  (client, &db, CONFIG_ROOT "/slowkeys_delay",
+		"*KRGSlowKeysDelayScale",	"AccessX*XmScale", 1000);
+	found |= xrm_get_int  (client, &db, CONFIG_ROOT "/bouncekeys_delay",
+		"*KRGDebounceScale",		"AccessX*XmScale", 1000);
+
+	/* Set the master enable flag last */
+	found |= xrm_get_bool (client, &db, CONFIG_ROOT "/enable",
+		"*EnableAccessXToggle.set",	"AccessX*ToggleButtonGadget.XmCSet");
+
+	if (!found) {
+		/* it would be nice to have a better message bu that would
+		 * break string freeze
+		 */
+		GtkWidget *warn = gtk_message_dialog_new (
+			gtk_window_get_transient_for (GTK_WINDOW (fsel)),
+			GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+			_("Unable to import AccessX settings from file '%s'"),
+			file);
+		g_signal_connect (warn,
+			"response",
+			G_CALLBACK (gtk_widget_destroy), NULL);
+		gtk_widget_show (warn);
+		return FALSE;
+	}
 	return TRUE;
 }
 
