@@ -37,20 +37,20 @@ dialog_response (GtkWidget *widget,
 		 gint       response_id,
 		 GConfChangeSet *changeset)
 {
-	if (response_id == GTK_RESPONSE_HELP) {
-		GError *error = NULL;
-
-		/* TODO : get this written */
-		gnome_help_display_desktop (NULL,
-			"accessibility-keyboard",
-			"accessibility-keyboard.xml",
-			"CONFIGURATION", &error);
-		if (error) {
-			g_warning ("help error: %s\n", error->message);
-			g_error_free (error);
-		}
-	} else
+	switch (response_id) {
+	case GTK_RESPONSE_HELP:
+		capplet_help (GTK_WINDOW (widget),
+			      "wgoscustaccess.xml",
+			      "goscustaccess-2");
+		break;
+	case GTK_RESPONSE_DELETE_EVENT:
+	case GTK_RESPONSE_CLOSE:
 		gtk_main_quit ();
+		break;
+	default:
+		/* Import CDE AccessX File */
+		break;
+	}
 }
 
 int
@@ -62,13 +62,14 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gnome_program_init (argv[0], VERSION, LIBGNOMEUI_MODULE, argc, argv,
+	gnome_program_init ("gnome-accessibility-keyboard-properties", VERSION,
+			    LIBGNOMEUI_MODULE, argc, argv,
 			    GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
 			    NULL);
 	activate_settings_daemon ();
 	
 	changeset = NULL;
-	dialog = setup_accessX_dialog (changeset, TRUE);
+	dialog = setup_accessX_dialog (changeset);
 	g_signal_connect (G_OBJECT (dialog),
 		"response",
 		G_CALLBACK (dialog_response), changeset);
