@@ -47,6 +47,8 @@ const KeyListEntry metacity_key_list[] =
   { "/apps/metacity/window_keybindings/activate_window_menu",      ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/toggle_fullscreen",         ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/toggle_maximized",          ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/maximize",                  ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/unmaximize",                ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/toggle_shaded",             ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/close",                     ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/minimize",                  ALWAYS_VISIBLE,  0 },
@@ -521,6 +523,31 @@ key_theme_changed (GConfClient *client,
 }
 
 
+static gboolean
+cb_check_for_uniqueness (GtkTreeModel *model,
+			 GtkTreePath  *path,
+			 GtkTreeIter  *iter,
+			 gpointer      user_data)
+{
+  KeyEntry *key_entry;
+  KeyEntry *tmp_key_entry;
+
+  key_entry = (KeyEntry *)user_data;
+  gtk_tree_model_get (key_entry->model, iter,
+		      KEYENTRY_COLUMN, &tmp_key_entry,
+		      -1);
+
+  if (tmp_key_entry != NULL &&
+      key_entry->keyval == tmp_key_entry->keyval &&
+      key_entry->mask   == tmp_key_entry->mask)
+    {
+      key_entry->editable = FALSE;
+      key_entry->gconf_key = tmp_key_entry->gconf_key;
+      return TRUE;
+    }
+  return FALSE;
+}
+
 static void
 accel_edited_callback (GtkCellRendererText *cell,
                        const char          *path_string,
@@ -533,7 +560,7 @@ accel_edited_callback (GtkCellRendererText *cell,
   GtkTreeModel *model;
   GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
   GtkTreeIter iter;
-  KeyEntry *key_entry;
+  KeyEntry *key_entry, tmp_key;
   GError *err = NULL;
   char *str;
 
@@ -545,7 +572,34 @@ accel_edited_callback (GtkCellRendererText *cell,
 
   /* sanity check */
   if (key_entry == NULL)
-    return;
+    {
+      gtk_tree_path_free (path);
+      return;
+    }
+
+  tmp_key.model  = model;
+  tmp_key.keyval = keyval;
+  tmp_key.mask   = mask;
+  tmp_key.editable = TRUE; /* kludge to stuff in a return flag */
+  gtk_tree_model_foreach (model, cb_check_for_uniqueness, &tmp_key);
+
+  /* flag to see if the new accelerator was in use by something */
+  if (!tmp_key.editable)
+    {
+      GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
+				       GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+				       GTK_MESSAGE_WARNING,
+				       GTK_BUTTONS_OK,
+				       _("That accelerator key is already in use by: %s\n"),
+				       key_entry->gconf_key);
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+
+      egg_cell_renderer_keys_set_accelerator (EGG_CELL_RENDERER_KEYS (cell),
+					      key_entry->keyval, key_entry->mask);
+      gtk_tree_path_free (path);
+      return;
+    }
 
   str = binding_name (keyval, mask, FALSE);
 
@@ -746,7 +800,9 @@ setup_dialog (GladeXML *dialog)
 
   renderer = (GtkCellRenderer *) g_object_new (EGG_TYPE_CELL_RENDERER_KEYS,
 					       "editable", TRUE,
+					       "accel_mode", EGG_CELL_RENDERER_KEYS_MODE_X,
 					       NULL);
+
   g_signal_connect (G_OBJECT (renderer),
 		    "keys_edited",
                     G_CALLBACK (accel_edited_callback),
