@@ -10,6 +10,7 @@
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "theme-common.h"
 #include "capplet-util.h"
@@ -17,7 +18,8 @@
 #include "gconf-property-editor.h"
 #include "file-transfer-dialog.h"
 
-#define GTK_THEME_KEY "/desktop/gnome/interface/gtk_theme"
+#define GTK_THEME_KEY	"/desktop/gnome/interface/gtk_theme"
+#define ICON_THEME_KEY	"/desktop/gnome/interface/icon_theme"
 
 #define MAX_ELEMENTS_BEFORE_SCROLLING 8
 
@@ -87,6 +89,7 @@ theme_selection_changed (GtkTreeSelection *selection,
       if (old_key && strcmp (old_key, new_key))
 	{
 	  gconf_client_set_string (client, GTK_THEME_KEY, new_key, NULL);
+	  gconf_client_set_string (client, ICON_THEME_KEY, new_key, NULL);
 	}
       g_free (old_key);
     }
@@ -413,19 +416,19 @@ drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 {
 	GList *uris;
 	GladeXML *dialog = data;
-	gchar *filename;
+	gchar *filename = NULL;
 
 	if (!(info == TARGET_URI_LIST || info == TARGET_NS_URL))
 		return;
 
 	uris = gnome_vfs_uri_list_parse ((gchar *) selection_data->data);
-
-	filename = gnome_vfs_uri_to_string (uris->data, GNOME_VFS_URI_HIDE_NONE);
-	if (strncmp (filename, "http://", 7) && strncmp (filename, "ftp://", 6))
-	{
-		g_free (filename);
-		filename = gnome_vfs_uri_to_string (uris->data, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+	if (uris != NULL && uris->data != NULL) {
+		GnomeVFSURI *uri = (GnomeVFSURI *) uris->data;
+		filename = gnome_vfs_unescape_string (
+			gnome_vfs_uri_get_path (uri), G_DIR_SEPARATOR_S);
+		gnome_vfs_uri_list_unref (uris);
 	}
+
 	gnome_file_entry_set_filename (GNOME_FILE_ENTRY (WID ("install_theme_picker")), filename);
 	g_free (filename);
 	gnome_vfs_uri_list_unref (uris);
